/*
 * I2C_LCD2x16.C
 *
 *  Created on: Oct 26, 2023
 *      Author: jm palomino
 *      
 */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "COPFree.h"
#include "PTB.h"
#include "TPM1_Delays.h"
#include "I2C_Software.h"
#include "I2C_LCD2x16.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
/* User includes (#include below this line is not maintained by Processor Expert) 

*********************************************************
TPM1_Delay(24)      =  47.6us                              
Cpu_Delay100US(1)   =  100 us                            
Cpu_Delay100US(100) =  10ms                           
Cpu_Delay100US(300) =  30ms                           
*********************************************************

*/
void I2C_LCD_Cmd(char out_char) {

    char hi_n, lo_n;
    char rs = 0x00;

    hi_n = out_char & 0xF0;
    lo_n = (out_char << 4) & 0xF0;

    TWI_Start();
    TWI_Write(LCD_ADDR);
    TWI_Write(hi_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(hi_n | rs | 0x00 | 0x08);
    Cpu_Delay100US(1);
    TWI_Write(lo_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(lo_n | rs | 0x00 | 0x08);
    TWI_Stop();

    if(out_char == 0x01) Cpu_Delay100US(20);
}
void I2C_LCD_Chr(char row, char column, char out_char) {

    char hi_n, lo_n;
    char rs = 0x01;

    switch(row){

        case 1:
        I2C_LCD_Cmd(0x80 + (column - 1));
        break;
        case 2:
        I2C_LCD_Cmd(0xC0 + (column - 1));
        break;
        case 3:
        I2C_LCD_Cmd(0x94 + (column - 1));
        break;
        case 4:
        I2C_LCD_Cmd(0xD4 + (column - 1));
        break;
    };

    hi_n = out_char & 0xF0;
    lo_n = (out_char << 4) & 0xF0;

    TWI_Start();
    TWI_Write(LCD_ADDR);
    TWI_Write(hi_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(hi_n | rs | 0x00 | 0x08);
    Cpu_Delay100US(100);
    TWI_Write(lo_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(lo_n | rs | 0x00 | 0x08);
    TWI_Stop();
}

void I2C_LCD_Chr_Cp(char out_char) {

    char hi_n, lo_n;
    char rs = 0x01;

    hi_n = out_char & 0xF0;
    lo_n = (out_char << 4) & 0xF0;

    TWI_Start();
    TWI_Write(LCD_ADDR);
    TWI_Write(hi_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(hi_n | rs | 0x00 | 0x08);
    Cpu_Delay100US(100);
    TWI_Write(lo_n | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(lo_n | rs | 0x00 | 0x08);
    TWI_Stop();
}

void I2C_LCD_Init() {

    char rs = 0x00;

    TWI_Start();
    TWI_Write(LCD_ADDR);
    Cpu_Delay100US(300);

    TWI_Write(0x30 | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(0x30 | rs | 0x00 | 0x08);
    Cpu_Delay100US(100);

    TWI_Write(0x30 | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(0x30 | rs | 0x00 | 0x08);
    Cpu_Delay100US(100);

    TWI_Write(0x30 | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(0x30 | rs | 0x00 | 0x08);
    Cpu_Delay100US(100);

    TWI_Write(0x20 | rs | 0x04 | 0x08);
    TPM1_Delay(24);
    TWI_Write(0x20 | rs | 0x00 | 0x08);
    TWI_Stop();
    Cpu_Delay100US(100);

    I2C_LCD_Cmd(0x28);
    I2C_LCD_Cmd(0x06);
}

void I2C_LCD_Out(char row, char col, char *text) {
    while(*text)
         I2C_LCD_Chr(row, col++, *text++);
}
